@ECHO OFF
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Programa:		Batch bridge de execucao de arquivos via SPEL (USD)
:: Modulo:			z_executar_bridge.cmd
:: Objetivo:		Utilizada pela funcao em SPEL 'z_executar' para executar comandos ou executaveis
::					a partir de triggers ou automaes de workflow na ferramenta USD. Esta rotina e
::					a funcao em SPEL foram desenvolvidas para fazer com que a chamada do metodo nativo
::					'exec' (SPEL) se torne sincrono e alem disso seja possivel receber como retorno da
::					execucao a saida da execucao e o codigo de erro gerado pelo executavel informado.
:: Autor:			Daniel Becker Bighelini
:: Criado em:		11/06/2015
:: Modificado por:	Daniel Becker Bighelini
:: Modificado em:	15/10/2015
:: Funcionamento:	
::                  1) Esta rotina eh disparada a partir da funcao 'z_executar' armazenada no arquivo
::					   'z_functions' na pasta NX_ROOT\SITE\MODS\MAJIC e deve ficar gravada dentro da
::					   pasta NX_ROOT\BIN da USD.
::					2) Na chamada sao informados dois parametros:
::					   #PAR1 = Local e nome do comando ou arquivo executavel
::					   #PAR2 = Local e nome do arquivo que sera utilizado para armazenar a saida da execucao.
::					3) Depois de um tratamento nas strings dos dois parametros, o comando/arquivo eh
::					   executado e a saida de execucao eh armazenado num arquivo intermediario na mesma pasta
::					   que deveria ser gerado final de saida. Alem disso na ultima linha do arquivo eh
::					   registrado o codigo ERRORLEVEL de execucao do comando/executavel utilizando como
::					   separador a string @ERRORLEVEL@;
::					4) Por fim o arquivo intermediario de saida eh copiado para a unidade mapeada referenciada
::					   no parametro 2.
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Habilita a expansao das variaveis de ambiente
SETLOCAL ENABLEDELAYEDEXPANSION

:: Salva pasta atual
PUSHD "%~dp0"

:: Verifica os parametros
SET batargc=0
FOR %%? IN (%*) DO (
	IF DEFINED batdebug ECHO %DATE% %TIME%	DEBUG		#ARG !batargc!: %%?
	SET /A batargc+=1
)
IF DEFINED batdebug ECHO %DATE% %TIME%	DEBUG		Quant. de parametros informados: %batargc%
ECHO.

:inicio
:: Define variaveis globais
CALL:atualiza_data_hora
SET batauthor=Daniel Becker Bighelini
SET batfunction=BAT BRIDGE VIA SPEL
SET batversion=1.01
SET batdescription=BRIDGE DE EXECUCAO DE ARQUIVOS VIA SPEL USD
SET batstart=%DATE% %TIME%
SET batuser=%USERNAME%
SET batname=%~n0
SET batext=%~x0
SET batfile=%batname%%batext%
SET batpath=%~dp0
SET battemp="%TEMP%\%batname%_%batagoraf%.tmp"
SET batpathfull="%batpath%%batname%%batext%"
FOR %%? IN (%batpathfull%) DO SET batdate=%%~t?
SET batlogname=%batname%.log
SET batlognameold=%batlogname%.old
SET batdebug=0
SET batlog=1
CALL:define_log

:: Define variaveis personalizaveis
SET batcmd=%1

:: Identifica o ultimo parametro informado
CALL:ultimo_parametro %*
SET batout=%batlastarg%

FOR %%? IN (%batout%) DO SET batserver=%%~dp?
:: Remove a ultima contrabarra do endereco remoto
SET batserver=%batserver:~0,-1%
SET batadmin=USUARIO
SET batpassword=SENHA
:: Captura em uma variavel apenas o nome do arquivo que sera gerado sem o caminho
FOR %%? IN (%batout%) DO SET batfileouttemp="%%~n?%%~x?"

:: Substitui duas aspas duplas por uma so
SET batcmd=%batcmd:""="%

:: Remove a primeira e a ultima aspas do comando
SET "batcmd=%batcmd:*"=%
:: Fim das variaveis personalizaveis

:: INICIO DA EXECUCAO
CALL:exibe_cabecalho
CALL:bloco_inicio

IF DEFINED batdebug (
	:: Capturando variaveis de ambiente do SCRIPT
	IF DEFINED batname (
		IF DEFINED batdebug CALL:logmsg DEBUG "Capturando variaveis utilizadas pelo SCRIPT..."
		FOR /F "tokens=1-2 delims==" %%a IN ('SET bat') DO (
			ECHO %DATE% %TIME%	DEBUG		%%a=%%b>>%batlogpath%
		)
	) ELSE (
		CALL:logmsg WARNING "AVISO: As variaveis de ambiente do script nao puderam ser coletadas."
	)
)

:: Executa o arquivo informado como primeiro parametro e armazena a SAIDA DA EXECUCAO
IF DEFINED batdebug CALL:logmsg DEBUG "Executando o comando/arquivo '%batcmd%'..."
CALL %batcmd%>%battemp%

:: REGISTRA no arquivo informado no parametro o RETURN CODE
SET batcmderrorlevel=!ERRORLEVEL!
ECHO @ERRORLEVEL@!batcmderrorlevel!>>%battemp%

IF DEFINED batdebug ECHO %DATE% %TIME%	DEBUG		ERRORLEVEL da execucao: !batcmderrorlevel!>>%batlogpath%

:: COPIA o arquivo intermediario com o nome do arquivo temporario de saida
IF EXIST %battemp% (
	IF DEFINED batdebug CALL:logmsg DEBUG "Mapeando unidade de rede no caminho %batserver%..."
	NET USE %batserver% /user:LOCALHOST\%batadmin% %batpassword% >NUL
	IF ERRORLEVEL 1 GOTO erro2
	
	IF DEFINED batdebug CALL:logmsg DEBUG "Copiando o arquivo temporario %battemp% como %batout%..."
	COPY %battemp% %batserver%\%batfileouttemp%

	IF DEFINED batdebug CALL:logmsg DEBUG "Removendo unidade mapeada..."
	NET USE %batserver% /DELETE /YES >NUL

) ELSE (
	CALL:erro "%batcmd%"
)

:: EXCLUI o arquivo temporario caso ele existir
IF EXIST %battemp% (
	IF DEFINED batdebug CALL:logmsg DEBUG "Excluindo o arquivo temporario %battemp% caso ele ainda exista..."
	DEL %battemp%
)

:: Caso a rotina nao tenha gerado nenhum erro, encerra a execucao
IF EXIST %batout% (
	GOTO sucesso
) ELSE (
	GOTO erro1
)
GOTO:EOF

:erro1
SET baterror_code=1
CALL:logmsg ERROR "ERRO [%baterror_code%] ao copiar arquivo %battemp% para o destino %batout%."
CALL:bloco_fim
EXIT /B %baterror_code%
GOTO:EOF

:erro2
SET baterror_code=2
CALL:logmsg ERROR "ERRO [%baterror_code%] ao mapear unidade de rede para o destino %batserver%."
CALL:bloco_fim
EXIT /B %baterror_code%
GOTO:EOF


:atualiza_data_hora
:: Consultando data atual
FOR %%A IN (%DATE%) DO (
    FOR /F "tokens=1-3 delims=/-" %%B in ("%%~A") DO (
        SET batdataf=%%D%%C%%B
        SET batdatas=%%B/%%C/%%D
    )
)
:: Consultando hora atual
FOR /F "tokens=1-3 delims=:.," %%A IN ("%TIME%") DO (
	SET bathoraf=%%A%%B%%C
	SET bathoras=%%A:%%B:%%C
)
:: Concatenando data e hora
SET batagoraf=%batdataf%%bathoraf%
SET batagoras=%batdatas% %bathoras%
GOTO:EOF

:exibe_cabecalho
:: Exibe o cabecalho do programa
IF DEFINED batdebug (
	ECHO *************************************************************
	ECHO %batfunction% %batversion% [%batfile%]
	ECHO Desenvolvido por %batauthor% em %batdate%
	ECHO Descricao: %batdescription%
	ECHO *************************************************************
)
GOTO:EOF

:logmsg
SET batmsgtype=%1
SET batmsgtext=%2
:: Removendo aspas do inicio e fim da string
SET "batmsgtype=%batmsgtype:*"=%
SET "batmsgtext=%batmsgtext:*"=%
SET batmsg=%DATE% %TIME%	%batmsgtype%		%batmsgtext%
IF "%batmsgtype%"=="SIGNIFICANT" SET batmsg=%DATE% %TIME%	%batmsgtype%	%batmsgtext%
IF DEFINED batdebug ECHO %batmsg%
IF DEFINED batlog (
	ECHO %batmsg%>>%batlogpath%
) ELSE (
	REM Registra sempre todos os tipos de erro
	IF "%batmsgtype%"=="ERROR" ECHO %batmsg%>>%batlogpath%
	IF "%batmsgtype%"=="WARNING" ECHO %batmsg%>>%batlogpath%
	IF "%batmsgtype%"=="FATAL" ECHO %batmsg%>>%batlogpath%
	IF "%batmsgtype%"=="CRITICAL" ECHO %batmsg%>>%batlogpath%
	IF "%batmsgtype%"=="SEVERE_ERROR" ECHO %batmsg%>>%batlogpath%
)
GOTO:EOF

:erro
SET baterror_code=%ERRORLEVEL%
SET baterror_msg=%1
SET baterror_msg=%baterror_msg:"='%
CALL:logmsg ERROR "ERRO [%baterror_code%] ao executar rotina '%baterror_msg%'."
EXIT /B %baterror_code%
GOTO:EOF

:sucesso
CALL:logmsg SIGNIFICANT	"Operacao CONCLUIDA com sucesso."
GOTO fim

:cancelar
CALL:logmsg SIGNIFICANT	"Operacao CANCELADA pelo usuario."
GOTO fim

:define_log
:: Define os arquivos que serao utilizados para registros de log
IF DEFINED NX_ROOT (
	IF DEFINED batdebug ECHO %DATE% %TIME%	DEBUG		Definindo local para armazenamento de logs na USD...
	SET batlogpath="%NX_ROOT%\log\!batlogname!"
	SET batlogpathold="%NX_ROOT%\log\!batlognameold!"
	:: Substitui a barra por contrabarra no path do servicedesk
	SET batfindbar=/
	SET batreplacebar=\
	CALL SET batlogpath=%%batlogpath:!batfindbar!=!batreplacebar!%%
	CALL SET batlogpathold=%%batlogpathold:!batfindbar!=!batreplacebar!%%
) ELSE (
	IF DEFINED batdebug ECHO %DATE% %TIME%	DEBUG		Definindo local para armazenamento de logs na pasta do script...
	SET batlogpath="!batpath!!batlogname!"
	SET batlogpathold="!batpath!!batlognameold!"
)
IF DEFINED batdebug CALL:logmsg DEBUG "Arquivo de logs utilizado: %batlogpath%"
IF DEFINED NX_LOGFILE_LIMIT (
	SET batlogsizemax=%NX_LOGFILE_LIMIT%
) ELSE (
	SET batlogsizemax=5000000
)
IF DEFINED batdebug CALL:logmsg DEBUG "Tamanho maximo do log: %batlogsizemax% bytes."
IF EXIST %batlogpath% CALL:verifica_log
GOTO:EOF

:verifica_log
:: Verifica se o arquivo de log atingiu o limite de tamanho definido
:: E em caso positivo, rotaciona o log.
IF DEFINED batdebug CALL:logmsg DEBUG "Verificando tamanho do arquivo de log..."
FOR /F "usebackq" %%A IN ('%batlogpath%') DO set batlogsize=%%~zA
IF DEFINED batdebug CALL:logmsg DEBUG "Tamanho atual do arquivo de log: %batlogsize%"
IF %batlogsize% GEQ %batlogsizemax% (
	IF DEFINED batdebug CALL:logmsg DEBUG "O arquivo de log ATINGIU o limite de tamanho de %batlogsizemax% bytes."
	IF DEFINED batdebug CALL:logmsg DEBUG "Removendo logs antigos e rotacionando log..."
	IF EXIST "%batlognameold%" DEL "%batlognameold%"
	IF EXIST "%batlogname%" IF NOT EXIST "%batlognameold%" REN "%batlogname%" "%batlognameold%"
)
GOTO:EOF

:ultimo_parametro
:: Identifica o ultimo parametro informado
SET "batlastarg=%~1"
SHIFT
IF NOT "%~1"=="" GOTO ultimo_parametro
GOTO:EOF

:bloco_inicio
:: Exibe o bloco de inicio do programa
IF DEFINED batdebug CALL:logmsg DEBUG "INICIO"
IF DEFINED batparameters (
	IF DEFINED batdebug (
		CALL:logmsg DEBUG "Linha de comando utilizada: %CMDCMDLINE%"
	)
)
GOTO:EOF

:bloco_fim
:: Exibe o bloco de fim do programa
IF DEFINED batdebug CALL:logmsg DEBUG "FIM"
GOTO:EOF

:fim
:: Reseta variaveis
CALL:bloco_fim
POPD
ENDLOCAL